@@ -3,7 +3,9 @@
 from datetime import datetime
 from decimal import Decimal as D
 from hashlib import sha1
+from pprint import pformat
 
+from ofxstatement import exceptions
 
 TRANSACTION_TYPES = [
     "CREDIT",       # Generic credit
@@ -33,7 +35,16 @@ ACCOUNT_TYPE = [
 ]
 
 
-class Statement(object):
+# Inspired by "How to print instances of a class using print()?"
+# on stackoverflow.com
+class Printable:
+    def __repr__(self):  # pragma: no cover
+        # do not set width to 1 because that makes the output really ugly
+        return "<" + type(self).__name__\
+            + "> " + pformat(vars(self), indent=4)
+
+
+class Statement(Printable):
     """Statement object containing statement items"""
     lines = None
 
@@ -57,8 +68,19 @@ class Statement(object):
         self.currency = currency
         self.account_type = account_type
 
+    def assert_valid(self):  # pragma: no cover
+        if not(self.start_balance is None or self.end_balance is None):
+            total_amount = sum(sl.amount for sl in self.lines)
 
-class StatementLine(object):
+            msg = "Start balance ({0}) plus the total amount ({1}) \
+should be equal to the end balance ({2})".format(self.start_balance,
+                                                total_amount,
+                                                 self.end_balance)
+            if self.start_balance + total_amount != self.end_balance:
+                raise exceptions.ValidationError(msg, self)
+
+
+class StatementLine(Printable):
     """Statement line data.
 
     All fields are initialized with some sample data so that field type may be
@@ -103,7 +125,7 @@ class StatementLine(object):
         self.check_no = None
         self.refnum = None
 
-    def __str__(self):
+    def __str__(self):  # pragma: no cover
         return """
         ID: %s, date: %s, amount: %s, payee: %s
         memo: %s
@@ -120,8 +142,9 @@ class StatementLine(object):
         if self.bank_account_to:
             self.bank_account_to.assert_valid()
 
+        assert(self.id or self.check_no or self.refnum)
 
-class BankAccount(object):
+class BankAccount(Printable):
     """Structure corresponding to BANKACCTTO and BANKACCTFROM elements from OFX
 
     Open Financial Exchange uses the Banking Account aggregate to identify an
@@ -161,11 +184,56 @@ def generate_transaction_id(stmt_line):
     """
     h = sha1()
     h.update(stmt_line.date.strftime("%Y-%m-%d %H:%M:%S").encode("utf8"))
-    h.update(stmt_line.memo.encode("utf8"))
-    h.update(str(stmt_line.amount).encode("utf8"))
+    if stmt_line.memo is not None:
+        h.update(stmt_line.memo.encode("utf8"))
+    if stmt_line.amount is not None:
+        h.update(str(stmt_line.amount).encode("utf8"))
     return h.hexdigest()
 
 
+def generate_unique_transaction_id(stmt_line, unique_id_set: set):
+    """
+    Generate a unique transaction id.
+
+    A bit of background: the problem with these transaction id's is that
+    they do do not only have to be unique, they also have to stay the same
+    for the same transaction every time you generate the statement.  So
+    generating random ids will not work, even though they will be unique,
+    GnuCash or beancount will recognize these transaction as "new" if you
+    happen to generate and import the same statement twice or import two
+    statements with overlapping periods.
+
+    The function generate_transaction_id() is deterministic, but does not
+    necesserily generate an unique id.
+
+    Therefore this function improves on it since you can create a
+    really unique id by adding an increment to the generated id (a string)
+    and keep on incrementing till it succeeds.
+
+    These are the steps:
+    1) supply a unique id set you want to use for checking uniqueness
+    2) next you generate an initial id by calling
+       generate_transaction_id()
+    3) assign the initial id to the current id (id)
+    4) increment a counter while the current id is a member of the set and
+       add the counter to the initial id and assign that to the current id
+    5) add the current id to the unique id set
+    6) return a list of the current id and the counter (if not 0)
+
+    The counter is returned in order to enable the caller to modify
+    its statement line, for example the memo field.
+    """
+    # Save the initial id
+    id = initial_id = generate_transaction_id(stmt_line)
+    counter = 0
+    while id in unique_id_set:
+        counter += 1
+        id = initial_id + str(counter)
+
+    unique_id_set.add(id)
+    return id + ('' if counter == 0 else '-' + str(counter))
+
+
 def recalculate_balance(stmt):
     """Recalculate statement starting and ending dates and balances.
 
